/* 
 * Please read the COPYRIGHT notice included at the top level
 */
%{

    /*
     * doctor.y,v 1.1.1.1 1996/07/18 22:39:56 arsharma Exp
     * This is the input to bison and has the parser
     * The grammar has been taken from the microsoft assembler manual
     */

#include "doctor_t.h"
#include "mnemonics_def.h"
#include "lex.h"
#include "symtab.h"
#include "memref.h"
#include "instr.h"
#include "write.h"
#include "registers.h"

#include <stdio.h>
extern char yytext[];
enum addr_mode;

/* This section has all the global variables used by doctor */

/*
 * Index into the output buffer, pointing to the end of       the previous
 * instruction
 */
unsigned start_pos;

/* See the documentation for instr_t->start_instr in instr.h */
unsigned start_instr;

/* indicates the start and end of the first operand  */
unsigned start_first, end_first;

/*
 * indicates the start and end of the second operand in       the output
 * buffer
 */
unsigned start_second, end_second;

/*
 * Indicates the number       of the current instruction,
 * within the basic block
 */
unsigned instr_num = 0;

/* used for register usage analysis */
unsigned reg_mask = 0;

/*
 * used       to count the static clock cycles since the last update
 * Reset when the thread clock is updated
 */
unsigned static_execution_time = 0;

/* number of operands in an instruction */
int num_ops;

/*
 * this       is an array of pointers, each one pointing to a record,
 * corresponding to an asm instruction
 */
instr_t **basic_block;

instr_t *current;		/* the current instruction */

/* True if we're at the end of a basic block */
bool_t end_block;

/* True if we're doing verification */
bool_t default_data_movement = FALSE;

/* True if we're just counting the memory references i.e. -c flag */
bool_t count_references = FALSE;

/* Toggled by aug_on and aug_off */
bool_t augment_on = TRUE;

/* Augment even stack references */
bool_t stack_ref_augment = TRUE;

/* Instructions normally don't have locks */
bool_t lock = FALSE;

/* no repeat prefix */
bool_t repeat_prefix = FALSE;

/* Are we just counting instructions ? */
bool_t instr_count_only = FALSE;

%}

%token AH
%token AL
%token ASTERISK
%token AUG_OFF
%token AUG_ON
%token AX
%token BH
%token BL
%token BP
%token BX
%token CB
%token CH
%token CL
%token COLON
%token COMMENT
%token CONSTANT
%token COMMA
%token CR0
%token CR2
%token CR3
%token CS
%token CX
%token DB0
%token DB1
%token DB2
%token DB3
%token DB6
%token DB7
%token DH
%token DI
%token DIRECTIVE
%token DL
%token DOLLAR
%token DOT
%token DS
%token DX
%token EAX
%token EBP
%token EBX
%token ECX
%token EDI
%token EDX
%token ES
%token ESI
%token ESP
%token FS
%token GS
%token ID
%token LABELDEF
%token LOCK
%token MNEMONIC
%token NEWLINE
%token OB
%token REP
%token REPE
%token REPZ
%token REPNE
%token REPNZ
%token SI
%token SP
%token SQOB
%token SQCB
%token SS
%token ST0
%token ST1
%token ST2
%token ST3
%token ST4
%token ST5
%token ST6
%token ST7
%token TR6
%token TR7
%token VALUE

%union {
    enum addr_modes y_mod; /* addressing mode */
    enum addr_modes *y_mods;
    int	 y_int;
}

%type <y_mod>  e01 e02 register	expr  disp disp01 disp02
%type <y_mods> asmInstruction

%%


program		: /* nothing */
		|  program line

line		: directive tail
		| LABELDEF
		  {
		      end_block	= TRUE;
		      if (current) {
			  current->end_block = TRUE;
			  flush_basic_block();
		      }
		  }
		| tail
		| instruction tail
		| AUG_ON
		  {
		      augment_on = TRUE;
		  }
		| AUG_OFF
		  {
		      augment_on = FALSE;
		  }

tail		: opt_comment NEWLINE;

instruction	: opt_instrPrefix asmInstruction
		  {
		    extern int yylineno;
		    extern mnemonic_table_t *last_mnem;

		    /* allocate	memory for the current instr */
		    current = new_instruction();

		    /* put info	into it	*/
		    current->source_line = yylineno;
		    current->modes = $2;
		    current->start_text	= start_pos;
		    current->end_text =	save_pos();
		    current->start_instr = start_instr;
		    current->start_first = start_first;
		    current->end_first = end_first;
		    current->start_second = start_second;
		    current->end_second	= end_second;
		    current->mnemonic =	last_mnem;
		    current->reg_mask =	reg_mask;
		    current->num_ops = num_ops;
		    current->augment = augment_on;
		    current->lock = lock;
		    current->rep = repeat_prefix;
		    /*
		     * this flag tells if the next instruction is a
		     * new basic block
		     */
		    end_block =
		      isjumper(current->mnemonic->representation);
		    current->end_block = end_block;

		    if (end_block)
		       flush_basic_block();

		    /* reset the register mask */
		    reg_mask = 0;
		    start_pos =	save_pos();
		  };

opt_instrPrefix	: instrPrefix tail
		| /* null */
		  {
			lock = FALSE;
			repeat_prefix =	FALSE;
		  }
		;


asmInstruction	: MNEMONIC
		  {
			$$ = NULL;
			num_ops	= 0;
		  }
		| MNEMONIC expr
		  {
			$$ = (enum addr_modes *) malloc(2 * sizeof(int));
			$$[0] =	$2;
			$$[1] =	UNKNOWN;
			end_first = save_pos();
			num_ops	= 1;
		  }
		| MNEMONIC expr	COMMA expr opt_third_operand
		  {
		      /* return	an array containing the	addressing */
		      /* modes of the operands */

			$$ = (enum addr_modes *) malloc(2 * sizeof(int));
			$$[0] =	$2;
			$$[1] =	$4;

			if (!end_second)
				/* in the presence of a third operand, end_second is
				 * already set! */
				end_second = save_pos();

			num_ops	= 2;
		  }
       ;

opt_third_operand : /* null */
                  | COMMA expr
                   /* some instructions like IMULL have a third
                    * operand, but that is never a memref and hence
                    * ignored */
                  ;

instrPrefix	: REP
		  {
			repeat_prefix =	TRUE;
		  }
		| REPE
		  {
			repeat_prefix =	TRUE;
		  }
		| REPZ
		  {
			repeat_prefix =	TRUE;
		  }
		| REPNE
		  {
			repeat_prefix =	TRUE;
		  }
		| REPNZ
		  {
			repeat_prefix =	TRUE;
		  }
		| LOCK
		  {
			lock = TRUE;
		  }
		;

expr		: e02
		  {
		      $$ = $1;
		  }
		;

e02		: e01
		  {
		      $$ = $1;
		  }
		| segmentRegister COLON	e01
		  {
		      $$ = $3;
		  }
		| ASTERISK e01
		  {
		      $$ = $2;
		  }
		;


e01		: disp OB register CB
		  {
		      if ($3 ==	STACKREG)
			  $$ = STACKREF;
		      else
			  $$ = INDIRECT;
		  }
		| disp OB COMMA	CONSTANT CB
		  {
		      $$ = INDIRECT;
		  }
		| disp OB COMMA	register COMMA CONSTANT	CB
		  {
		      $$ = INDIRECT;
		  }
		| disp OB register COMMA  register COMMA CONSTANT
		  CB
		  {
		      $$ = INDIRECT;
		  }
		| disp OB register COMMA register CB
		  {
		      $$ = DIRECT;
		  }
		| disp02
		  {
		      $$ = $1;
		  }
		| VALUE
		  {
		      $$ = IMMEDIATE;
		  }
		| segmentRegister
		  {
		    $$ = REG;
		  }
		| register
		  {
		      $$ = $1;
		  }
		;

disp		: disp02
		  {
		      $$ = $1;
		  }
		| /* null */
		  {
		      $$ = UNKNOWN;
		  }

disp02		: disp01
		  {
		      $$ = $1;
		  }
		| disp01 CONSTANT
		  {
		      $$ = $1;
		  }
		;

disp01		: CONSTANT
		  {
		      $$ = IMMEDIATE;
		  }
		| ID quirks
		  {
		      extern mnemonic_table_t *last_mnem;

		      if (isjumper(last_mnem->representation))
			  $$= IMMEDIATE;
		      else
			  $$ = DIRECT;
		  }
		| DOT ID
		  {
		      $$ = IMMEDIATE;
		  }
		;

quirks		: /* null */
		| DOT CONSTANT
		;

register	: specialRegister
		  {
		      $$ = REG;
		  }
		| gpRegister
		  {
		      $$ = REG;
		  }
		| byteRegister
		  {
		      $$ = REG;
		  }
		| stackRegister
		  {
		      $$ = STACKREG;
		  };

specialRegister	: CR0
		| CR2
		| CR3
		| DB0
		| DB1
		| DB2
		| DB3
		| DB6
		| DB7
		| TR6
		| TR7;

gpRegister	: AX
		  {
		      reg_mask |= EAX_MASK;
		  }
		| EAX
		  {
		      reg_mask |= EAX_MASK;
		  }
		| BX
		  {
		      reg_mask |= EBX_MASK;
		  }
		| EBX
		  {
		      reg_mask |= EBX_MASK;
		  }
		| CX
		  {
		      reg_mask |= ECX_MASK;
		  }
		| ECX
		  {
		      reg_mask |= ECX_MASK;
		  }
		| DX
		  {
		      reg_mask |= EDX_MASK;
		  }
		| EDX
		  {
		      reg_mask |= EDX_MASK;
		  }
		| BP
		  {
		      reg_mask |= EBP_MASK;
		  }
		| EBP
		  {
		      reg_mask |= EBP_MASK;
		  }
		| DI
		  {
		      reg_mask |= EDI_MASK;
		  }
		| EDI
		  {
		      reg_mask |= EDI_MASK;
		  }
		| SI
		  {
		      reg_mask |= ESI_MASK;
		  }
		| ESI
		  {
		      reg_mask |= ESI_MASK;
		  }
		| ST0
		| ST1
		| ST2
		| ST3
		| ST4
		| ST5
		| ST6
		| ST7
		;

stackRegister	: SP
		  {
		      reg_mask |= ESP_MASK;
		  }
		| ESP
		  {
		      reg_mask |= ESP_MASK;
		  }
		;

byteRegister	: AL
		  {
		      reg_mask |= EAX_MASK;
		  }
		| AH
		  {
		      reg_mask |= EAX_MASK;
		  }
		| BL
		  {
		      reg_mask |= EBX_MASK;
		  }
		| BH
		  {
		      reg_mask |= EBX_MASK;
		  }
		| CL
		  {
		      reg_mask |= ECX_MASK;
		  }
		| CH
		  {
		      reg_mask |= ECX_MASK;
		  }
		| DL
		  {
		      reg_mask |= EDX_MASK;
		  }
		| DH
		  {
		      reg_mask |= EDX_MASK;
		  }
		;

segmentRegister	: CS
		  {
		      reg_mask |= CS_MASK;
		  }
		| DS
		  {
		      reg_mask |= DS_MASK;
		  }
		| ES
		  {
		      reg_mask |= ES_MASK;
		  }
		| FS
		  {
		      reg_mask |= FS_MASK;
		  }
		| GS
		  {
		      reg_mask |= GS_MASK;
		  }
		| SS
		  {
		      reg_mask |= SS_MASK;
		  }
		;

opt_comment	: /* null */
		| COMMENT;

directive	: DIRECTIVE
		  ;


%%

#include "write.h"
#include <unistd.h>

extern FILE *yyin;
char input_filename[MAX_ID_LENGTH];
int token_count = 0;

extern char *optarg;
/* this might need changes across platforms */
extern int optind;

/* doesn't support any options yet */
int
parse_cmd_line (int argc, char **argv)
{
    int c, errflg;

    errflg = 0;
    optind = 1;

    while ((c = getopt (argc, argv, "Vcghi")) != EOF)
	switch (c) {
	    case 'V':
		default_data_movement = TRUE;
		break;
	    case 'c':
		count_references = TRUE;
		break;
	    case 'g':
		stack_ref_augment = FALSE;
		break;
            case 'i':
                instr_count_only = TRUE;
                break;
	    case 'h':
		usage ();
		break;
	    case '?':
		errflg++;
		break;
	    default:
		break;
	}

    if ((default_data_movement) && (stack_ref_augment))
	yyerror ("Data movement	is not supported for stack references\n"
		 "You should turn the -g flag on\n");

    if (errflg)
	usage ();
}

void
usage ()
{
    fprintf (stderr, "usage: doctor [<option> ... ] [filename]\n"
	     "Options:\n"
	     "\t-V: Turns data movement	on\n"
	     "\t-c: Instrument to count	the number of memory references\n"
	     "\t-g: Instrument only global references\n"
             "\t-i: Print dynamic instruction count only\n"
	     "\t-h: Print this help screen\n");
    exit (1);
}

main (int argc, char *argv[])
{
    extern int yylineno;
    extern char yytext[];

    parse_cmd_line (argc, argv);

    /* the last one is the input file name.. */

    if (optind == argc - 1)
	strcpy (input_filename, (const char *) argv[optind]);
    else if (argc > optind)
	fprintf (stderr, "doctor: warning: junk	at the end of command"
		 " line\n");

    yyinit ();

#if YYDEBUG != 0
    yydebug = 1;
#endif

    fprintf (stdout, "/	This code is generated by doctor \n");

    yyparse ();
    cleanup ();

    return 0;

}
