/* 
 * Please read the COPYRIGHT notice included at the top level
 */
/*
 * write.c,v 1.1.1.1 1996/07/18 22:39:57 arsharma Exp
 * Purpose: To write out the augmentation and some string management
 *          routines
 * Abbreviations: OPB = output buffer
 */

#include <stdio.h>
#include <varargs.h>
#include <ctype.h>
#include <string.h>
#include <assert.h>

#include "doctor_t.h"
#include "lex.h"
#include "memref.h"
#include "instr.h"
#include "write.h"
#include "mnemonics_def.h"

/* variables to manage a circular buffer of characters */
int opb_head, opb_tail;
char *output_buffer;

extern yyerror ();

/* put a character into the buffer */
int
put_buf (char c)
{
    if (opb_tail + 1 == opb_head)
	yyerror ("token	buffer overflow");

    output_buffer[opb_tail] = c;
    opb_tail = INC_OPB_PTR (opb_tail);
}

/* flushes the buffer till till_here */
int
flush (int till_here)
{
    while (opb_head != till_here) {
	fprintf (stdout, "%c", output_buffer[opb_head]);
	opb_head = (opb_head + 1) % OPB_SIZE;
    }
}

/* skips the text in the buffer till_here */
int
skip (int till_here)
{
    opb_head = till_here;
}

/* allocate memory for the buffer */
int
alloc_output_buf ()
{
    if (!(output_buffer = (char *) malloc (OPB_SIZE * sizeof (char))))
	  yyerror ("Not enough memory	for the	token buffer");
}

/* returns a pointer to the tail of the buffer */
int
save_pos ()
{
    return opb_tail;
}

/* returns a pointer to the beginning  of the current line in the buffer */
int
start_line ()
{
    int i;

    i = opb_tail - 1;

    while ((i != opb_head) && (output_buffer[i] != '\n'))
	i = DEC_OPB_PTR (i);

    if (i == opb_head)
	return opb_head;
    else
	return INC_OPB_PTR (i);

}

/* returns a pointer to the end of the line, containing point */
int
end_line (int point)
{
    int i;

    i = point;

    while ((i != opb_tail) && (output_buffer[i] != '\n'))
	i = INC_OPB_PTR (i);

    return INC_OPB_PTR (i);

}

int aug_end = 0;
extern char *aug_buffer;

/*
 * Does       all the augmentation. Handles formatted writes. Keeps
 * appending the string supplied to the augmented buffer
 */

void
augment (va_alist)
     va_dcl
{
    va_list argp;
    char *fmt;

    va_start (argp);
    fmt = va_arg (argp, char *);
    vsprintf (aug_buffer + aug_end, fmt, argp);
    va_end (argp);

    /* update aug_end */
    aug_end += strlen (aug_buffer + aug_end);

    assert (aug_end < AUG_BUF_SIZE);
}

/* called when the pre instruction augmentation is done */
void
augment_done (instr_t *instr)
{
    ref_t *ref1, *ref2;

    aug_buffer[aug_end++] = '\0';

    /* set up the pointer to augment buffer */
    instr->post_aug_start = aug_buffer + aug_end;

    /* Do some updating depending on what the instruction does */
    ref1 = instr->mem_ref[0];
    ref2 = instr->mem_ref[1];

    /* If the instruction writes the stack, update the stack_offset */
    if ((ref1) && (ref1->write) && (ref1->stack))
	ref1->stack_offset += instr->mnemonic->operand_size;

    if ((ref2) && (ref2->write) && (ref2->stack))
	ref2->stack_offset -= instr->mnemonic->operand_size;
}

/* Called at the end of basic blocks */
int
flush_aug ()
{
    aug_buffer[0] = '\0';
    aug_end = 0;
}
