/* 
 * Please read the COPYRIGHT notice included at the top level
 */
/*
 * Definitions of exported global variables.
 *
 * Copyright (C) 1993 by Jack E. Veenstra (veenstra@cs.rochester.edu)
 * 
 * This file is part of MINT, a MIPS code interpreter and event generator
 * for parallel programs.
 * 
 * MINT is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version.
 * 
 * MINT is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with MINT; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef __export_h
#define __export_h

#include "event.h"
#include "macros.h"

/* The maximum number of processes that can be simulated. This is used
 * to allocate space. This value can be changed with the "-p" command
 * line option up to a maximum of MAXPROC.
 */
extern int Max_nprocs;

/* The absolute maximum number of processes that can be simulated. */
#define MAXPROC 256

/* the number of existing processes */
extern int Nprocs;

/* the maximum process id ever used (process ids start at zero) */
extern int Maxpid;

/* The file descriptor used by MINT for its output. Can be assigned by
 * the user program. Defaults to stderr. If NULL, no output from MINT
 * is generated.
 */
extern FILE *Mint_output;

/* Ckpoint_freq controls the frequency of calls to sim_checkpoint().
 * This can be set on the command line as well as changed by the user while
 * the program is running. If set to zero, no further calls to sim_checkpoint()
 * will occur.
 */
extern unsigned int Ckpoint_freq;

/* If true, then the simulator must supply a value on a read by using
 * the pointer "value" in the event structure to write the value that
 * should be used. On a write, the "value" pointer points to the value
 * being written by the object program. This flag is set by the "-V"
 * command line option.
 */
extern int Verify_protocol;

/* The time of the first fork, or zero if no forks yet */
extern mint_time_t First_fork_elapsed;

/* The accumulated cpu time of processor 0 at the time of the first fork */
extern mint_time_t First_fork_cpu;

#endif /* !__export_h */
